#!/bin/bash
###############################################################
#
#     Format:   s3-detecttime.sh module_name file_name
# 
#  - this script write to file_name the timestamps for tags 
#    from a CVS repository
#
# Prerequisites:
# - a CVS repository located in ./cvsrepo
#
###############################################################

module="" 
filename=""
tag=""
pwd=$PWD

export CVSROOT=$PWD/cvsrepo
cvsroot=$CVSROOT

export TZ=GMT

# Test if the module name is passed as a parameter
if [ $# != 2 ] ; then
   echo "Usage: s3-detecttime.sh module_name file_name"
   exit 1
else
   module=$1 
   filename=$2  
fi

# init work dir
workdir=$pwd/tmp-detecttime
rm -rf $workdir
mkdir $workdir

###############################################################
#                      STAGE 1                                #
#                                                             #
# Get the tag list from the CVS repository                    #
#                                                             #
###############################################################

# Find CVS tags (not starting with "H") 
# <module-option>
excludedtags='^H'

rm -f $workdir/cvstags.txt
cvs -Q rlog -h $module | \
   awk -F"[.:]" '/^\t/&&$(NF-1)!=0{print $1}' | \
   sort -u  | \
   while read tagName
   do
      echo $tagName
   done | \
   grep -v $excludedtags >> $workdir/cvstags.txt


###############################################################
#                      STAGE 2                                #
#                                                             #
# Detect timestamps for tags ...                              #
#                                                             #
###############################################################


echo "Detect timestamps for tags ..."
rm -f $workdir/cvsts.txt

for line in $(awk '{print $1}' $workdir/cvstags.txt); do   
   # Checkout the current tag from CVS
   tag=$line
   

   echo "CVS checkout $tag ..."   
   rm -fr $workdir/t-m10
   mkdir $workdir/t-m10
   cd $workdir/t-m10
   
   cvs -Q export -r $tag $module
   cd $workdir

   # Detect last modification time
   filem=$(find ./t-m10/$module/ -type f -printf "%T@\0%p\0" | awk '
      {
         if ($0>max) {
               max=$0; 
               getline mostrecent
         } else 
               getline
      } 
      END{print mostrecent} ' RS='\0')
   
   
   # last modification date
   lmd=$(stat -c %Y $filem)
   
   # GMT BZR date
   
   # The ":date" option in bazaar means:
   #  "Matches the first entry after a given date"
   # The commit time is later(or equal) to this(lmd) date
   # Add 1 sec to avoid the equal case 
   blmd=$((lmd+1))
   # The bazzar revision could be found with
   # before:date:blmd
   # TODO : last revision case

   vdatenext=$(date -d "1970-01-01 + $blmd secs" +%F)
   vtimenext=$(date -d "1970-01-01 + $blmd secs" +%T)
      
   vdate=$(date -d "1970-01-01 + $lmd secs" +%F)
   vtime=$(date -d "1970-01-01 + $lmd secs" +%T)

   echo " " $vdate $vtime $filem
   echo $tag $vdate $vtime $vdatenext $vtimenext $filem >> $workdir/cvsts.txt
done
cd $pwd
sort $workdir/cvsts.txt > $filename